@@ -67,6 +67,7 @@ struct _MetaWindow
   
   MetaDisplay *display;
   MetaScreen *screen;
+  const MetaMonitorInfo *monitor;
   MetaWorkspace *workspace;
   Window xwindow;
   /* may be NULL! not all windows get decorated */
@@ -160,6 +161,11 @@ struct _MetaWindow
    */
   guint on_all_workspaces : 1;
 
+  /* This is true if the client requested sticky, and implies on_all_workspaces == TRUE,
+   * however on_all_workspaces can be set TRUE for other internal reasons too, such as
+   * being override_redirect or being on the non-primary monitor. */
+  guint on_all_workspaces_requested : 1;
+
   /* Minimize is the state controlled by the minimize button */
   guint minimized : 1;
   guint tab_unminimized : 1;
@@ -247,11 +253,9 @@ struct _MetaWindow
   /* EWHH demands attention flag */
   guint wm_state_demands_attention : 1;
   
-  /* this flag tracks receipt of focus_in focus_out and
-   * determines whether we draw the focus
-   */
+  /* this flag tracks receipt of focus_in focus_out */
   guint has_focus : 1;
-  
+
   /* Have we placed this window? */
   guint placed : 1;
 
@@ -383,6 +387,9 @@ struct _MetaWindow
   MetaGroup *group;
 
   GObject *compositor_private;
+
+  /* Focused window that is (directly or indirectly) attached to this one */
+  MetaWindow *attached_focus_window;
 };
 
 struct _MetaWindowClass
@@ -625,5 +632,10 @@ void meta_window_update_icon_now (MetaWindow *window);
 
 void meta_window_update_role (MetaWindow *window);
 void meta_window_update_net_wm_type (MetaWindow *window);
+void meta_window_update_monitor (MetaWindow *window);
+void meta_window_update_on_all_workspaces (MetaWindow *window);
+
+void meta_window_propagate_focus_appearance (MetaWindow *window,
+                                             gboolean    focused);
 
 #endif
