@@ -24,13 +24,14 @@
  */
 
 #include <config.h>
-#include "workspace.h"
+#include "screen-private.h"
+#include <meta/workspace.h>
 #include "workspace-private.h"
 #include "boxes-private.h"
-#include "errors.h"
-#include "prefs.h"
+#include <meta/errors.h>
+#include <meta/prefs.h>
 
-#include "compositor.h"
+#include <meta/compositor.h>
 
 #include <X11/Xatom.h>
 #include <string.h>
@@ -45,7 +46,6 @@ enum {
 };
 
 void meta_workspace_queue_calc_showing   (MetaWorkspace *workspace);
-static void set_active_space_hint        (MetaScreen *screen);
 static void focus_ancestor_or_mru_window (MetaWorkspace *workspace,
                                           MetaWindow    *not_this_one,
                                           guint32        timestamp);
@@ -557,7 +557,7 @@ meta_workspace_activate_with_focus (MetaWorkspace *workspace,
 
   workspace->screen->active_workspace = workspace;
 
-  set_active_space_hint (workspace->screen);
+  meta_screen_set_active_workspace_hint (workspace->screen);
 
   /* If the "show desktop" mode is active for either the old workspace
    * or the new one *but not both*, then update the
@@ -618,10 +618,20 @@ meta_workspace_activate_with_focus (MetaWorkspace *workspace,
    meta_screen_calc_workspace_layout (workspace->screen, num_workspaces,
                                       new_space, &layout2);
 
-   if (layout1.current_col < layout2.current_col)
-     direction = META_MOTION_RIGHT;
-   if (layout1.current_col > layout2.current_col)
-     direction = META_MOTION_LEFT;
+   if (meta_ui_get_direction() == META_UI_DIRECTION_RTL)
+     {
+       if (layout1.current_col > layout2.current_col)
+         direction = META_MOTION_RIGHT;
+       else if (layout1.current_col < layout2.current_col)
+         direction = META_MOTION_LEFT;
+     }
+   else
+    {
+       if (layout1.current_col < layout2.current_col)
+         direction = META_MOTION_RIGHT;
+       else if (layout1.current_col > layout2.current_col)
+         direction = META_MOTION_LEFT;
+    }
 
    if (layout1.current_row < layout2.current_row)
      {
@@ -742,32 +752,6 @@ meta_workspace_list_windows (MetaWorkspace *workspace)
   return workspace_windows;
 }
 
-static void
-set_active_space_hint (MetaScreen *screen)
-{
-  unsigned long data[1];
-
-  /* this is because we destroy the spaces in order,
-   * so we always end up setting a current desktop of
-   * 0 when closing a screen, so lose the current desktop
-   * on restart. By doing this we keep the current
-   * desktop on restart.
-   */
-  if (screen->closing > 0)
-    return;
-  
-  data[0] = meta_workspace_index (screen->active_workspace);
-
-  meta_verbose ("Setting _NET_CURRENT_DESKTOP to %lu\n", data[0]);
-  
-  meta_error_trap_push (screen->display);
-  XChangeProperty (screen->display->xdisplay, screen->xroot,
-                   screen->display->atom__NET_CURRENT_DESKTOP,
-                   XA_CARDINAL,
-                   32, PropModeReplace, (guchar*) data, 1);
-  meta_error_trap_pop (screen->display);
-}
-
 void
 meta_workspace_invalidate_work_area (MetaWorkspace *workspace)
 {
